<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output method="xml" version="1.0" encoding="UTF-8" standalone="no" indent="yes"/>	
	<xsl:param name="DecisionXML" />

	<!-- 
	***************************************************************************************************************************************************************************
	Global variable definitions
	***************************************************************************************************************************************************************************
	-->

	<!-- *** Format utility variables *** -->

	<xsl:variable name="newline">
	<xsl:text>
	</xsl:text>
	</xsl:variable>	

	<!-- 
	***************************************************************************************************************************************************************************
	Converts the collaboration template from version 4.1.1 to the latest.
	***************************************************************************************************************************************************************************
	-->


	<!-- *** XSLT starts at the root template *** -->
	<xsl:template match="/">
		<!-- *** write the DTD for our root element CollabTemplate *** -->
		<xsl:text disable-output-escaping="yes">
			&lt;!DOCTYPE CollabTemplate [
			&lt;!--
			 Base Elements and Attributes
			-->
			&lt;!ELEMENT Title (#PCDATA)>
			&lt;!ELEMENT Desc (#PCDATA)>
			&lt;!ELEMENT Value (#PCDATA)>
			&lt;!ELEMENT Default (#PCDATA)>
			&lt;!ELEMENT CreateTime (#PCDATA)>
			&lt;!ATTLIST CreateTime Format CDATA #REQUIRED>
			&lt;!ELEMENT ModifyTime (#PCDATA)>
			&lt;!ATTLIST ModifyTime Format CDATA #REQUIRED>
			&lt;!ELEMENT GUICoord EMPTY>
			&lt;!ATTLIST GUICoord X CDATA #REQUIRED>
			&lt;!ATTLIST GUICoord Y CDATA #REQUIRED>
			&lt;!ELEMENT GUICoord2 (GUICoord,GUICoord)>
			&lt;!ELEMENT GUICoords (GUICoord,GUICoord+)>
			&lt;!ELEMENT Color EMPTY>
			&lt;!ATTLIST Color R CDATA #REQUIRED>
			&lt;!ATTLIST Color G CDATA #REQUIRED>
			&lt;!ATTLIST Color B CDATA #REQUIRED>
			&lt;!ELEMENT GUIFont (FaceName,PointSize,Height,Color)>
			&lt;!ATTLIST GUIFont Italic CDATA #REQUIRED>
			&lt;!ATTLIST GUIFont UnderLine CDATA #REQUIRED>
			&lt;!ATTLIST GUIFont StrikeOut CDATA #REQUIRED>
			&lt;!ELEMENT FaceName (#PCDATA)>
			&lt;!ELEMENT PointSize (#PCDATA)>
			&lt;!ELEMENT Height (#PCDATA)>

			&lt;!--
			 Header section
			-->
			&lt;!ELEMENT Header (Title,Desc,CreateTime,ModifyTime)>

			&lt;!--
			 Collaboration Document
			-->
			&lt;!ELEMENT CollabTemplate (Header,TemplateDefinition,GUITemplateDefinition,Scenarios,Messages)>
			&lt;!ATTLIST CollabTemplate Name CDATA #REQUIRED>
			&lt;!ATTLIST CollabTemplate Version CDATA #REQUIRED>
			&lt;!ATTLIST CollabTemplate FormatVersion CDATA #REQUIRED>

			&lt;!--
			 Collaboration Definition section
			-->
			&lt;!ELEMENT TemplateDefinition (Desc,Properties,MinTransactionLevel,Package,GeneratedVars,BusObjRefs)>
			&lt;!ELEMENT MinTransactionLevel EMPTY>
			&lt;!ATTLIST MinTransactionLevel Level (None|Minimum|Best|Stringent) "None">
			&lt;!ELEMENT Package (#PCDATA)>
			&lt;!ELEMENT GeneratedVars (#PCDATA)>
			&lt;!ELEMENT BusObjRefs (BusObjRef)*>
			&lt;!ELEMENT BusObjRef (ConnectorName,Verbs)>
			&lt;!ATTLIST BusObjRef Name CDATA #REQUIRED>
			&lt;!ATTLIST BusObjRef BusObjType CDATA #REQUIRED>
			&lt;!ATTLIST BusObjRef IsRequired (Yes|No) #REQUIRED>
			&lt;!ATTLIST BusObjRef IsConfigured (Yes|No) #REQUIRED>
			&lt;!ELEMENT ConnectorName (#PCDATA)>
			&lt;!ELEMENT Verbs (Verb)*>
			&lt;!ELEMENT Verb (ScenarioName)>
			&lt;!ATTLIST Verb Name CDATA #REQUIRED>
			&lt;!ATTLIST Verb IsSubscribeEvent (Yes|No) #REQUIRED>
			&lt;!ELEMENT ScenarioName (#PCDATA)>

			&lt;!--
			 GUI Collaboration Definition section
			-->
			&lt;!ELEMENT GUITemplateDefinition (GUIProperties,GUIImports,GUIUserSection1,GUIUserSection2,GUIUserSection3,GUIVariables)>
			&lt;!ELEMENT GUIProperties (GeneralProperties?,BOProbeProperties?)>
			&lt;!ELEMENT GeneralProperties (GUIProperty)*>
			&lt;!ELEMENT BOProbeProperties (GUIProperty)*>
			&lt;!ELEMENT GUIProperty (GUIPropertyType,Desc,MaxStringLength,MaxMultipleValues,DefaultValues,PossibleValues)>
			&lt;!ATTLIST GUIProperty Name CDATA #REQUIRED>
			&lt;!ELEMENT GUIPropertyType (#PCDATA)>
			&lt;!ELEMENT MaxStringLength (#PCDATA)>
			&lt;!ELEMENT MaxMultipleValues (#PCDATA)>
			&lt;!ELEMENT DefaultValues (Values,Ranges)>
			&lt;!ELEMENT PossibleValues (Values,Ranges)>
			&lt;!ELEMENT Values (GUIPropertyValue)*>
			&lt;!ELEMENT Ranges (Range)*>
			&lt;!ELEMENT Range (Min,Max)*>
			&lt;!ELEMENT GUIPropertyValue (#PCDATA)>
			&lt;!ELEMENT Min (#PCDATA)>
			&lt;!ELEMENT Max (#PCDATA)>
			&lt;!ELEMENT GUIImports (GUIImport)*>
			&lt;!ELEMENT GUIImport (ImportName,Comment)>
			&lt;!ELEMENT ImportName (#PCDATA)>
			&lt;!ELEMENT Comment (#PCDATA)>
			&lt;!ELEMENT GUIUserSection1 (#PCDATA)>
			&lt;!ELEMENT GUIUserSection2 (#PCDATA)>
			&lt;!ELEMENT GUIUserSection3 (#PCDATA)>
			&lt;!ELEMENT GUIVariables (GUIVariable)*>
			&lt;!ELEMENT GUIVariable (VarType,ObjectType?,VarName,VarValue,FirstStepInit?,VarModifier1,VarModifier2,VarModifier3,Comment)>
			&lt;!ELEMENT VarType (#PCDATA)>
			&lt;!ELEMENT ObjectType (#PCDATA)>
			&lt;!ATTLIST ObjectType NeedsTwoStepInit (Yes|No) #REQUIRED>
			&lt;!ELEMENT FirstStepInit (#PCDATA)>
			&lt;!ELEMENT VarName (#PCDATA)>
			&lt;!ELEMENT VarValue (#PCDATA)>
			&lt;!ELEMENT VarModifier1 (#PCDATA)>
			&lt;!ELEMENT VarModifier2 (#PCDATA)>
			&lt;!ELEMENT VarModifier3 (#PCDATA)>

			&lt;!--
			 Scenario section
			-->
			&lt;!ELEMENT Scenarios (Scenario)*>
			&lt;!ELEMENT Scenario (Header,GUIDefinition,GUIDiagrams,ModelScenario)>
			&lt;!ATTLIST Scenario Name CDATA #REQUIRED>
			&lt;!ATTLIST Scenario Version CDATA #REQUIRED>
			&lt;!ATTLIST Scenario DiagramRef IDREF #REQUIRED>
			&lt;!ATTLIST Scenario DiagramRefGUI IDREF #REQUIRED>
			&lt;!ELEMENT GUIDefinition (Desc,GUIUserSection1,GUIUserSection2,GUIVariables)>

			&lt;!--
			 GUI Diagram section
			-->
			&lt;!ELEMENT GUIDiagrams (GUIDiagram)*>
			&lt;!ELEMENT GUIDiagram (Header,GUIProperties,GUISettings,NodesGUI,LinksGUI,Texts)>
			&lt;!ATTLIST GUIDiagram Name ID #REQUIRED>
			&lt;!ATTLIST GUIDiagram Version CDATA #REQUIRED>
			&lt;!ELEMENT ModelScenario (Diagram)*>

			&lt;!--
			 GUI Diagram Settings section
			-->
			&lt;!ELEMENT GUISettings (DiagramPermission,PageSettings,ComponentView,Grid)>
			&lt;!ELEMENT DiagramPermission EMPTY>
			&lt;!ATTLIST DiagramPermission Value (Yes|No) #REQUIRED>
			&lt;!ELEMENT PageSettings EMPTY>
			&lt;!ATTLIST PageSettings SizeX CDATA #REQUIRED>
			&lt;!ATTLIST PageSettings SizeY CDATA #REQUIRED>
			&lt;!ATTLIST PageSettings BoundVisible (Yes|No) #REQUIRED>
			&lt;!ELEMENT ComponentView EMPTY>
			&lt;!ATTLIST ComponentView Type (Yes|No) #REQUIRED>
			&lt;!ATTLIST ComponentView UIDs (Yes|No) #REQUIRED>
			&lt;!ATTLIST ComponentView Labels (Yes|No) #REQUIRED>
			&lt;!ELEMENT Grid (Color)>
			&lt;!ATTLIST Grid Visible (Yes|No) #REQUIRED>
			&lt;!ATTLIST Grid Snap (Yes|No) #REQUIRED>
			&lt;!ATTLIST Grid SizeX CDATA #REQUIRED>
			&lt;!ATTLIST Grid SizeY CDATA #REQUIRED>

			&lt;!--
			 GUI Node, Link and Text elements section
			-->
			&lt;!ELEMENT NodeLinkGUI (GUIFont)>
			&lt;!ATTLIST NodeLinkGUI UniqIDRef IDREF #REQUIRED>
			&lt;!ELEMENT NodesGUI (NodeGUI)*>
			&lt;!ELEMENT NodeGUI (NodeLinkGUI,GUICoord2,DecisionNode?)>
			&lt;!ATTLIST NodeGUI DiagramRefGUI IDREF #IMPLIED>
			&lt;!ELEMENT LinksGUI (LinkGUI)*>
			&lt;!ELEMENT LinkGUI (NodeLinkGUI,GUICoords,BOProbe?,Connect?,DecisionLink?)>
			&lt;!ELEMENT DecisionLink (Label,Desc)>
			&lt;!ATTLIST DecisionLink UniqID ID #REQUIRED>
			&lt;!ELEMENT BOProbe (BOUnit*)>
			&lt;!ATTLIST BOProbe Name CDATA #REQUIRED>
			&lt;!ATTLIST BOProbe LinkID IDREF #REQUIRED>
			&lt;!ATTLIST BOProbe ActionID IDREF #REQUIRED>
			&lt;!ATTLIST BOProbe To IDREF #REQUIRED>
			&lt;!ELEMENT BOUnit (#PCDATA)>
			&lt;!ATTLIST LinkGUI IsOrthogonal (Yes|No) #REQUIRED>
			&lt;!ELEMENT Texts (Text)*>
			&lt;!ELEMENT Text (TextValue,GUIFont,GUICoord2)>
			&lt;!ELEMENT TextValue (#PCDATA)>

			&lt;!--
			 ModelElement
			-->
			&lt;!ELEMENT ModelElement (Desc)>
			&lt;!ATTLIST ModelElement Version CDATA #REQUIRED>

			&lt;!--
			 ModelStructureElement
			-->
			&lt;!ELEMENT ModelStructureElement (ModelElement)>
			&lt;!ATTLIST ModelStructureElement Name CDATA #REQUIRED>

			&lt;!--
			 ModelActivityElement
			-->
			&lt;!ELEMENT ModelActivityElement (ModelElement,Label)>
			&lt;!ATTLIST ModelActivityElement UniqID ID #REQUIRED>
			&lt;!ELEMENT Label (#PCDATA)>

			&lt;!--
			 Misc - dummy elements
			-->
			&lt;!ELEMENT PseudoState (Node)>
			&lt;!ELEMENT ProcessStep (Node)>

			&lt;!--
			 Diagram
			-->
			&lt;!ELEMENT Diagram (ModelStructureElement,Declaration,DiagramBody)>
			&lt;!ATTLIST Diagram Name ID #REQUIRED>
			&lt;!ATTLIST Diagram Version CDATA #REQUIRED>
			&lt;!ELEMENT Declaration (Section1,Section2)>
			&lt;!ELEMENT Section1 (#PCDATA)>
			&lt;!ELEMENT Section2 (#PCDATA)>
			&lt;!ELEMENT DiagramBody (ModelActivityElement,Nodes,Links)>

			&lt;!--
			 Nodes - note that there can be only one start node
			-->
			&lt;!ELEMENT Node (ModelActivityElement)>
			&lt;!ATTLIST Node UserModified (Yes|No) "No">
			&lt;!ELEMENT Nodes ( (ActionNode|SubMapNode|SubDiagramNode|IteratorNode|ServiceNode|OutboundServiceNode|InboundServiceNode|SuccessEndNode|FailureEndNode|BreakNode|ForkNode|JoinNode)* |
			 (InitialNode, (ActionNode|SubMapNode|SubDiagramNode|IteratorNode|ServiceNode|OutboundServiceNode|InboundServiceNode|SuccessEndNode|FailureEndNode|BreakNode|ForkNode|JoinNode)*) |
			 ( (ActionNode|SubMapNode|SubDiagramNode|IteratorNode|ServiceNode|OutboundServiceNode|InboundServiceNode|SuccessEndNode|FailureEndNode|BreakNode|ForkNode|JoinNode)*,InitialNode) |
			 ( (ActionNode|SubMapNode|SubDiagramNode|IteratorNode|ServiceNode|OutboundServiceNode|InboundServiceNode|SuccessEndNode|FailureEndNode|BreakNode|ForkNode|JoinNode)*,InitialNode, (ActionNode|SubMapNode|SubDiagramNode|IteratorNode|ServiceNode|OutboundServiceNode|InboundServiceNode|SuccessEndNode|FailureEndNode|BreakNode|ForkNode|JoinNode)*))>
			&lt;!ELEMENT CompositeNode (ProcessStep)>
			&lt;!ATTLIST CompositeNode DiagramRef IDREF #REQUIRED>

			&lt;!ELEMENT InitialNode (PseudoState,CorrelationSet*)>
			&lt;!ELEMENT SuccessEndNode (PseudoState)>
			&lt;!ELEMENT FailureEndNode (PseudoState)>
			&lt;!ELEMENT BreakNode (PseudoState)>
			&lt;!ELEMENT ForkNode (PseudoState)>
			&lt;!ELEMENT JoinNode (PseudoState)>
			&lt;!ELEMENT ActionNode (ProcessStep,CodeFragment)>
			&lt;!ELEMENT SubMapNode (ActionNode)>
			&lt;!ATTLIST SubMapNode SubMapName CDATA #REQUIRED>
			&lt;!ATTLIST SubMapNode SubMapType CDATA #REQUIRED>
			&lt;!ELEMENT CodeFragment (#PCDATA)>
			&lt;!ATTLIST CodeFragment Type (Java|XML) #REQUIRED>
			&lt;!ELEMENT ServiceNode (ProcessStep,RegBusObjName,RegBusObjVerb,RegBusObjRefName,CmpBusObjName,CmpBusObjVerb,CmpBusObjRefName,WaitTimeOut,CorrelationSet*)>
			&lt;!ELEMENT RegBusObjName (#PCDATA)>
			&lt;!ELEMENT RegBusObjVerb (#PCDATA)>
			&lt;!ELEMENT RegBusObjRefName (#PCDATA)>
			&lt;!ELEMENT CmpBusObjName (#PCDATA)>
			&lt;!ELEMENT CmpBusObjVerb (#PCDATA)>
			&lt;!ELEMENT CmpBusObjRefName (#PCDATA)>
			&lt;!ELEMENT WaitTimeOut (#PCDATA)>
			&lt;!ATTLIST WaitTimeOut Type (NotUsed|Constant|Parameter|Variable) #REQUIRED>
			&lt;!ELEMENT CorrelationSet (CorrelationAttribute+)>
			&lt;!ATTLIST CorrelationSet IsInit (Yes|No) "No">
			&lt;!ELEMENT CorrelationAttribute (AttributeName,VariableName)>
			&lt;!ELEMENT AttributeName (#PCDATA)>
			&lt;!ELEMENT VariableName (#PCDATA)>
			&lt;!ELEMENT OutboundServiceNode (ServiceNode)>
			&lt;!ATTLIST OutboundServiceNode IsSync CDATA #REQUIRED>
			&lt;!ELEMENT InboundServiceNode (ServiceNode)>
			&lt;!ELEMENT SubDiagramNode (CompositeNode)>
			&lt;!ELEMENT IteratorNode (CompositeNode,IteratorVariable,ThruThisObject,Start,Condition,Increment)>
			&lt;!ATTLIST IteratorNode Type (Attribute|BusObj|Loop) #REQUIRED>
			&lt;!ELEMENT IteratorVariable (#PCDATA)>
			&lt;!ELEMENT ThruThisObject (#PCDATA)>
			&lt;!ELEMENT Start (#PCDATA)>
			&lt;!ELEMENT Condition (#PCDATA)>
			&lt;!ELEMENT Increment (#PCDATA)>
			&lt;!ELEMENT DecisionNode (Label,Desc,DecisionBranches)>
			&lt;!ATTLIST DecisionNode UniqID ID #REQUIRED>
			&lt;!ELEMENT DecisionBranch (Condition,Comment,Label,LinkID)>
			&lt;!ELEMENT DecisionBranches (DecisionBranch,DecisionBranch+)>
			&lt;!ATTLIST DecisionBranch Type (NotUsed|Normal|Default|Exception) #REQUIRED>
			&lt;!ELEMENT LinkID (#PCDATA)>

			&lt;!--
			 Links
			-->
			&lt;!ELEMENT Link (ModelActivityElement,Connect)>
			&lt;!ELEMENT Links (TransitionLink|MessageLink|ExceptionLink)*>
			&lt;!ELEMENT Connect EMPTY>
			&lt;!ATTLIST Connect From IDREF #REQUIRED>
			&lt;!ATTLIST Connect To IDREF #REQUIRED>

			&lt;!ELEMENT TransitionLink (Link,GuardCondition)>
			&lt;!ELEMENT GuardCondition (#PCDATA)>
			&lt;!ELEMENT ExceptionLink (Link,ExceptionName)>
			&lt;!ELEMENT ExceptionName (#PCDATA)>
			&lt;!ELEMENT MessageLink (Link)>

			&lt;!--
			 Misc properties stuff - not exactly belong to diagram, but we stick them here!
			-->
			&lt;!ELEMENT Properties (Property)*>
			&lt;!ELEMENT Property (Attributes)>
			&lt;!ATTLIST Property Name CDATA #REQUIRED>
			&lt;!ELEMENT Attributes (Attribute)*>
			&lt;!ELEMENT Attribute (Value,Desc)>
			&lt;!ATTLIST Attribute Name CDATA #REQUIRED>
			&lt;!ATTLIST Attribute Type CDATA #REQUIRED>

			&lt;!--
			 Messages Element
			-->
			&lt;!ELEMENT Messages (CommonMessages,SpecificMessages)>
			&lt;!ELEMENT CommonMessages (#PCDATA)>
			&lt;!ELEMENT SpecificMessages (Msg)*>
			&lt;!ELEMENT Msg (MsgBody, MsgExpl)>
			&lt;!ATTLIST Msg msg_key CDATA #REQUIRED>
			&lt;!ELEMENT MsgBody (#PCDATA)>
			&lt;!ELEMENT MsgExpl (#PCDATA)>
			&lt;!ATTLIST SpecificMessages locale CDATA #REQUIRED>

			]>

		</xsl:text>	

		<!-- *** Now write the real XML content for the root element CollabTemplate *** -->
		<xsl:apply-templates select="CollabTemplate" />	
	</xsl:template>

	<!-- *** the root template applies the CollabTemplate element *** -->
	<xsl:template match="CollabTemplate">
		<xsl:element name="{name()}">
		<xsl:call-template name="writeAllAttirbutes" />
		<xsl:value-of select="$newline" />
			<xsl:copy-of select="Header" />
			<xsl:value-of select="$newline" />
			
			<xsl:copy-of select="TemplateDefinition" />			
			<xsl:value-of select="$newline" />

			<xsl:copy-of select="GUITemplateDefinition" />			
			<xsl:value-of select="$newline" />

			<xsl:apply-templates select="Scenarios" />
			<xsl:value-of select="$newline" />

			<xsl:copy-of select="Messages" />						
			<xsl:value-of select="$newline" />
		</xsl:element>
	</xsl:template>

	<!-- *** the CollabTemplate template applies the Scenarios element *** -->
	<xsl:template match="Scenarios" >
		<xsl:element name="{name()}">
		<xsl:call-template name="writeAllAttirbutes" />
		<xsl:value-of select="$newline" />
			<xsl:apply-templates select="Scenario" />
		</xsl:element>	
	</xsl:template>

	<!-- *** the CollabTemplate template applies the Scenarios element *** -->
	<xsl:template match="Scenario" >
		<xsl:variable name="ScenarioName" select="@Name" />
		<xsl:element name="{name()}">
		<xsl:call-template name="writeAllAttirbutes" />
		<xsl:value-of select="$newline" />

			<xsl:copy-of select="Header" />
			<xsl:value-of select="$newline" />

			<xsl:copy-of select="GUIDefinition" />
			<xsl:value-of select="$newline" />

			<xsl:choose>
				<xsl:when test="document($DecisionXML)/*/Scenario" >					
					<xsl:call-template name="writeGUIDiagrams">
						<xsl:with-param name="ScenarioName" select="$ScenarioName" />
					</xsl:call-template>

					<xsl:value-of select="$newline" />

					<xsl:call-template name="writeModelScenario">
						<xsl:with-param name="ScenarioName" select="$ScenarioName" />
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:copy-of select="GUIDiagrams" />
					<xsl:value-of select="$newline" />
					<xsl:copy-of select="ModelScenario" />
				</xsl:otherwise>
			</xsl:choose>
			<xsl:value-of select="$newline" />
		</xsl:element>
	</xsl:template>

	<!-- 
	***************************************************************************************************************************************************************************
	Utility template functions.
	***************************************************************************************************************************************************************************
	-->

	<!-- *** copies all source element's attribute name and value pairs to the destination element *** -->
	<xsl:template name="writeAllAttirbutes">
		<xsl:for-each select="@*">	
			<xsl:attribute name="{name()}">
				<xsl:value-of select="." />
			</xsl:attribute>
		</xsl:for-each>
	</xsl:template>



	<!-- *** 
		Write the GUIDiagrams element of Scenario element 	    
	 *** -->
	<xsl:template name="writeGUIDiagrams" >
	<xsl:param name="ScenarioName" />

		<GUIDiagrams>
		<xsl:value-of select="$newline" />
			<xsl:for-each select="GUIDiagrams/GUIDiagram">
				<xsl:call-template name="writeGUIDiagram">
					<xsl:with-param name="ScenarioName" select="$ScenarioName" />
				</xsl:call-template>
				<xsl:value-of select="$newline" />
			</xsl:for-each>
		</GUIDiagrams>

	</xsl:template>

	<!-- *** 
		Write the GUIDiagram element of GUIDiagrams element 	    
	 *** -->
	<xsl:template name="writeGUIDiagram">	
	<xsl:param name="ScenarioName" />
		
		<xsl:variable name="strGUIDiagramName" select="@Name" />
		<xsl:element name="{name()}">
		<xsl:call-template name="writeAllAttirbutes" />
		<xsl:value-of select="$newline" />
			<xsl:copy-of select="Header" />
			<xsl:value-of select="$newline" />

			<xsl:copy-of select="GUIProperties" />

			<xsl:copy-of select="GUISettings" />
			<xsl:value-of select="$newline" />

			<xsl:variable name="nodeDecisionDiagram" select="document($DecisionXML)/*/Scenario[@Name = $ScenarioName]/Diagram[@GUIName = $strGUIDiagramName]" />			

			<xsl:call-template name="writeNodesGUI">
				<xsl:with-param name="nodeDecisionDiagram" select="$nodeDecisionDiagram" />
			</xsl:call-template>
			<xsl:value-of select="$newline" />

			<xsl:call-template name="writeLinksGUI">
				<xsl:with-param name="nodeDecisionDiagram" select="$nodeDecisionDiagram" />
			</xsl:call-template>
			<xsl:value-of select="$newline" />

			<xsl:copy-of select="Texts" />
			<xsl:value-of select="$newline" />
		</xsl:element>
	</xsl:template>

	<!-- *** 
		Write the NodesGUI element of GUIDiagram element 	    
	 *** -->
	<xsl:template name="writeNodesGUI" >
	<xsl:param name="nodeDecisionDiagram" />

		<NodesGUI>
		<xsl:value-of select="$newline" />
			<xsl:for-each select="NodesGUI/NodeGUI">
				<xsl:choose>
					<xsl:when test="DecisionNode">
						<xsl:call-template name="writeDecisionNodeGUI">
							<xsl:with-param name="nodeDecisionDiagram" select="$nodeDecisionDiagram" />
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:copy-of select="." />					
					</xsl:otherwise>
				</xsl:choose>
				<xsl:value-of select="$newline" />
			</xsl:for-each>
		</NodesGUI>

	</xsl:template>

	<!-- *** 
		Write the NodeGUI element of NodesGUI element 	    
	 *** -->
	<xsl:template name="writeDecisionNodeGUI">	
	<xsl:param name="nodeDecisionDiagram" />

		<xsl:element name="{name()}">
		<xsl:value-of select="$newline" />
			<xsl:copy-of select="NodeLinkGUI" />
			<xsl:value-of select="$newline" />

			<xsl:copy-of select="GUICoord2" />
			<xsl:value-of select="$newline" />

			<xsl:call-template name="writeDecisionNode">
				<xsl:with-param name="nodeDecisionDiagram" select="$nodeDecisionDiagram" />
			</xsl:call-template>
		</xsl:element>

	</xsl:template>

	<xsl:template name="writeDecisionNode">	
	<xsl:param name="nodeDecisionDiagram" />

		<xsl:variable name="NodeID" select="NodeLinkGUI/@UniqIDRef" />
		<xsl:variable name="nodeDecision" select="$nodeDecisionDiagram/DecisionLinkNode[@UniqID = $NodeID]" />
		<DecisionNode>
			<xsl:attribute name="UniqID">
				<xsl:value-of select="$NodeID" />
			</xsl:attribute>

			<Label>
				<xsl:value-of select="$nodeDecision/Label" />
			</Label>

			<Desc>
				<xsl:value-of select="$nodeDecision/Desc" />
			</Desc>

			<DecisionBranches>
				<xsl:for-each select="DecisionNode/DecisionBranch">
					<xsl:copy-of select="." />
					<xsl:value-of select="$newline" />					
				</xsl:for-each>
			</DecisionBranches>
			
		</DecisionNode>

	</xsl:template>

	<!-- *** 
		Write the LinksGUI element of GUIDiagram element 	    
	 *** -->
	<xsl:template name="writeLinksGUI" >
	<xsl:param name="nodeDecisionDiagram" />

		<LinksGUI>
		<xsl:value-of select="$newline" />
			<xsl:for-each select="LinksGUI/LinkGUI">
				<xsl:variable name="nLinkID" select="NodeLinkGUI/@UniqIDRef" />
				<xsl:variable name="nodeSourceLink" select="$nodeDecisionDiagram/DecisionLinkNode[*]/SourceLink[@UniqID = $nLinkID]" />
				<xsl:variable name="nodeTargetLink" select="$nodeDecisionDiagram/DecisionLinkNode[*]/DestinationLinks/DestLink[@UniqID = $nLinkID]" />
				<xsl:choose>
					<xsl:when test="$nodeSourceLink">
						<xsl:call-template name="writeSourceDecisionLinkGUI">
							<xsl:with-param name="nodeDecisionLink" select="$nodeSourceLink" />
						</xsl:call-template>
					</xsl:when>
					<xsl:when test="$nodeTargetLink">
						<xsl:call-template name="writeTargetDecisionLinkGUI">
							<xsl:with-param name="nodeDecisionLink" select="$nodeTargetLink" />
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:copy-of select="." />
					</xsl:otherwise>

				</xsl:choose>
				<xsl:value-of select="$newline" />
			</xsl:for-each>
		</LinksGUI>

	</xsl:template>

	<!-- *** 
		Write the LinkGUI element of LinksGUI element 	    
	 *** -->
	<xsl:template name="writeSourceDecisionLinkGUI">	
	<xsl:param name="nodeDecisionLink" />

		<xsl:element name="{name()}">
		<xsl:call-template name="writeAllAttirbutes" />
		<xsl:value-of select="$newline" />
			<xsl:copy-of select="NodeLinkGUI" />
			<xsl:value-of select="$newline" />

			<xsl:copy-of select="GUICoords" />
			<xsl:value-of select="$newline" />

			<xsl:if test="$nodeDecisionLink" >
				<xsl:element name="Connect">
					<xsl:attribute name="To">
						<xsl:value-of select="$nodeDecisionLink/@To" />
					</xsl:attribute>
					<xsl:attribute name="From">
						<xsl:value-of select="$nodeDecisionLink/@From" />
					</xsl:attribute>					
				</xsl:element>
				<xsl:element name="DecisionLink">
					<xsl:attribute name="UniqID">
						<xsl:value-of select="$nodeDecisionLink/@UniqID" />
					</xsl:attribute>
					<Label>
						<xsl:value-of select="$nodeDecisionLink/Label" />
					</Label>
					<Desc>
						<xsl:value-of select="$nodeDecisionLink/Desc" />						
					</Desc>
				</xsl:element>
			</xsl:if>
			

		</xsl:element>		
	</xsl:template>

	<xsl:template name="writeTargetDecisionLinkGUI">	
	<xsl:param name="nodeDecisionLink" />

		<xsl:element name="{name()}">
		<xsl:call-template name="writeAllAttirbutes" />
		<xsl:value-of select="$newline" />
			<xsl:copy-of select="NodeLinkGUI" />
			<xsl:value-of select="$newline" />

			<xsl:copy-of select="GUICoords" />
			<xsl:value-of select="$newline" />

			<xsl:if test="$nodeDecisionLink" >
				<xsl:element name="Connect">
					<xsl:attribute name="To">
						<xsl:value-of select="$nodeDecisionLink/@To" />
					</xsl:attribute>
					<xsl:attribute name="From">
						<xsl:value-of select="$nodeDecisionLink/@From" />
					</xsl:attribute>					
				</xsl:element>
			</xsl:if>
			

		</xsl:element>		
	</xsl:template>


	<!-- *** 
		Write the ModelScenario element of Scenario element 	    
	 *** -->
	<xsl:template name="writeModelScenario" >
	<xsl:param name="ScenarioName" />

		<ModelScenario>
		<xsl:value-of select="$newline" />
			<xsl:for-each select="ModelScenario/Diagram">
				<xsl:call-template name="writeDiagram" >
					<xsl:with-param name="ScenarioName" select="$ScenarioName"/>
				</xsl:call-template>
				<xsl:value-of select="$newline" />
			</xsl:for-each>
		</ModelScenario>
	</xsl:template>

	<!-- *** 
		Write the Diagram element of ModelScenario element 	    
	 *** -->
	<xsl:template name="writeDiagram">	
	<xsl:param name="ScenarioName" />

		<xsl:variable name="strDiagramName" select="@Name" />

		<xsl:element name="{name()}">
		<xsl:call-template name="writeAllAttirbutes" />
		<xsl:value-of select="$newline" />
			<xsl:copy-of select="ModelStructureElement" />
			<xsl:value-of select="$newline" />

			<xsl:copy-of select="Declaration" />
			<xsl:value-of select="$newline" />

			<xsl:variable name="nodeDecisionDiagram" select="document($DecisionXML)/*/Scenario[@Name = $ScenarioName]/Diagram[@Name = $strDiagramName]" />			

			<DiagramBody>
			<xsl:value-of select="$newline" />
				<xsl:copy-of select="DiagramBody/ModelActivityElement" />
				<xsl:value-of select="$newline" />

				<xsl:call-template name="writeNodes">
					<xsl:with-param name="nodeDecisionDiagram" select="$nodeDecisionDiagram" />					
				</xsl:call-template>
				<xsl:value-of select="$newline" />

				<xsl:call-template name="writeLinks">
					<xsl:with-param name="nodeDecisionDiagram" select="$nodeDecisionDiagram" />					
				</xsl:call-template>
				<xsl:value-of select="$newline" />
			</DiagramBody>
			<xsl:value-of select="$newline" />
		</xsl:element>

	</xsl:template>


	<!-- *** 
		Write the Nodes element of DiagramBody element 	    
	 *** -->
	<xsl:template name="writeNodes" >
	<xsl:param name="nodeDecisionDiagram" />

		<Nodes>
		<xsl:value-of select="$newline" />
			<xsl:for-each select="DiagramBody/Nodes/*">
				<xsl:choose>
					<xsl:when test="name()='ActionNode'">
						<xsl:variable name="nNodeID" select="ProcessStep/Node/ModelActivityElement/@UniqID" />	
						<xsl:choose>							
							<xsl:when test="$nodeDecisionDiagram/DecisionLinkNode[@UniqID = $nNodeID]">	
							</xsl:when>
							<xsl:otherwise>						
								<xsl:copy-of select="." /> 						
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>						
						<xsl:copy-of select="." /> 						
						<xsl:value-of select="$newline" />
					</xsl:otherwise>
				</xsl:choose>				

			</xsl:for-each>
		</Nodes>
	</xsl:template>

	<!-- *** 
		Write the Links element of DiagramBody element 	    
	 *** -->
	<xsl:template name="writeLinks" >
	<xsl:param name="nodeDecisionDiagram" />

		<Links>
		<xsl:value-of select="$newline" />
			<xsl:for-each select="DiagramBody/Links/*">
				<xsl:variable name="nLinkID" select="Link/ModelActivityElement/@UniqID" />
				<xsl:variable name="nodeSourceLink" select="$nodeDecisionDiagram/DecisionLinkNode[*]/SourceLink[@UniqID = $nLinkID]" />
				<xsl:variable name="nodeTargetLink" select="$nodeDecisionDiagram/DecisionLinkNode[*]/DestinationLinks/DestLink[@UniqID = $nLinkID]" />
				<xsl:choose>
					<xsl:when test="$nodeSourceLink">
					</xsl:when>
					<xsl:when test="$nodeTargetLink">
						<xsl:call-template name="writeTargetDecisionLink">
							<xsl:with-param name="nodeDecisionLink" select="$nodeTargetLink" />
						</xsl:call-template>
					</xsl:when>
					<xsl:otherwise>
						<xsl:copy-of select="." />
					</xsl:otherwise>
				</xsl:choose>				
			</xsl:for-each>
		</Links>
	</xsl:template>

	<xsl:template name="writeTargetDecisionLink">	
	<xsl:param name="nodeDecisionLink" />

		<xsl:element name="{name()}">
		<xsl:call-template name="writeAllAttirbutes" />
		<xsl:value-of select="$newline" />
			<Link>
				<xsl:copy-of select="Link/ModelActivityElement" />
				<xsl:value-of select="$newline" />

				<xsl:choose>
					<xsl:when test="$nodeDecisionLink" >
						<xsl:element name="Connect">
							<xsl:attribute name="To">
								<xsl:value-of select="Link/Connect/@To" />
							</xsl:attribute>
							<xsl:attribute name="From">
								<xsl:value-of select="$nodeDecisionLink/../../SourceLink/@From" />
							</xsl:attribute>					
						</xsl:element>
					</xsl:when>
					<xsl:otherwise>
						<xsl:copy-of select="Connect" />					
					</xsl:otherwise>
				</xsl:choose>
				<xsl:value-of select="$newline" />
			</Link>


			<xsl:copy-of select="GuardCondition" />
			<xsl:value-of select="$newline" />

			<xsl:copy-of select="ExceptionName" />
			<xsl:value-of select="$newline" />			

		</xsl:element>		
	</xsl:template>

</xsl:stylesheet>
